function [model predictions] = trainAndTestSingleExemplar(test_panorama, panoramas_base, num_negatives, bounds, panoramas)

scale = 0.2;
panoramas = applyPanoramaBounds(panoramas, bounds);

fprintf(1, 'Generating positive examples.\n');
[positives pmap] = generatePositiveExamples(panoramas, 1);
sample = zeros(2,1);
sampled_panorama = -1;
for i=1:size(panoramas, 2)
   if (size(find(pmap(i).map), 1) ~= 0) 
       [x y] = find(pmap(i).map);
       sample(1) = x; sample(2) = y;
       sampled_panorama = i;
   end
end
figure; 
hold on; 
    imshow(imresize(imread([panoramas_base '/' panoramas(sampled_panorama).panoid]), scale)); 
    rectangle('Position', [sample(2) sample(1) 5 5]); 
hold off;

fprintf(1, 'Generating negative examples.\n');
[negatives nmap] = generateNegativeExamples(panoramas, num_negatives);

training_data = [positives;negatives];
labels = [ones(size(positives,1),1); -ones(size(negatives,1),1)];

fprintf('Extracting features from input image.\n');
%% SANITY TESTING ONLY
test_panorama = [panoramas_base '/' panoramas(sampled_panorama).panoid];
%%
[f features_string] = getFeatures(imresize(imread(test_panorama), scale));
features = reshape(f, size(f,1) * size(f,2), size(f,3));

predictions = zeros(size(features, 1), 1);

fprintf(1, 'Training model (SVM).\n');
[type type_string] = getType(6);
model = train(labels, sparse(training_data), ['-c 10 -s ' type ' -e 0.01']);
clear training_data labels positives negatives;

% Run model against the panorama.
fprintf(1, 'Computing predictions from model.\n');
[labels, accuracy, prob_estimates] = predict(randi([0,1],size(features,1),1), sparse(features), model, '-b 1');

w = model.w;
for i=1:size(predictions,1)
    predictions(i) = dot(features(i,:), w);
end
if (max(predictions(:)) ~= 0)
    predictions = predictions / max(predictions(:));
end
figure; imshow(makePredictionOverlay(imresize(imread(test_panorama), scale), predictions, ''));

% Create visualization.
figure;
hold on;
    image = imresize(imread(test_panorama), scale);
    imshow(image);
    
    labels = reshape(labels, size(image,1), size(image,2));
    locations = find(labels == 1);
    for i=1:size(locations,1)
        [x y] = ind2sub([size(image,1) size(image,2)], locations(i));
        rectangle('Position', [y x 5 5]);
    end
hold off;

