#include <cppconn/driver.h>
#include <cppconn/exception.h>
#include <cppconn/resultset.h>
#include <cppconn/statement.h>
#include <cppconn/prepared_statement.h>
#include <common/types.h>
#include <gflags/gflags.h>
#include <glog/logging.h>
#include <map>
#include <mysql_connection.h>
#include <mysql_driver.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <util/assert.h>
#include <vector>

DEFINE_string(hostname, "ravi-server1", "The hostname where the mySQL database is located.");
DEFINE_string(database, "silicon", "The name of the database holding the training data.");
DEFINE_string(database_user, "silicon", "The user to use for connecting to the database.");
DEFINE_string(database_password, "0012259c5c", "The password to use for connecting to the database.");
DEFINE_string(table, "salience_test", "The table in the database that has the trials.");
DEFINE_string(annotations_table, "salience_test_annotations", "The table in the database that has the annotations.");
DEFINE_string(experiment, "saliency-test.onebox.text-annotated", 
	      "The name of the experiment to use "
	      "(run 'SELECT expid FROM table GROUP BY expid' to see list of valid ids).");
DEFINE_string(output_file, "output.dat", "The output filename");

using std::map;
using std::string;
using std::vector;

struct Annotations {
  int32 x;
  int32 y;
  int32 width;
  int32 height;
};

typedef map<string, vector<Annotations> > TrainingData;

void SaveTrainingData(const TrainingData& training_data, const string& output_file) {
  FILE* fid = fopen(output_file.c_str(), "w");
  if (!fid) {
    LOG(WARNING) << "Could not open output file: " << output_file;
    return;
  }

  fprintf(fid, "%ld\n", static_cast<int32>(training_data.size()));

  // Print out the list of panoids.
  for (TrainingData::const_iterator iter = training_data.begin(); iter != training_data.end(); iter++) {
    fprintf(fid, "%s\n", (*iter).first.c_str());
  }

  // Print out the annotations.
  for (TrainingData::const_iterator iter = training_data.begin(); iter != training_data.end(); iter++) {
    const uint32 M = (*iter).second.size();
    fprintf(fid, "%ld\n", static_cast<int32>(M));
    for (uint32 i = 0; i < M; i++) {
      fprintf(fid, "%ld\n", (*iter).second[i].x);
      fprintf(fid, "%ld\n", (*iter).second[i].y);
      fprintf(fid, "%ld\n", (*iter).second[i].width);
      fprintf(fid, "%ld\n", (*iter).second[i].height);
    }
  }

  fclose(fid);
}

int main(int argc, char** argv) {
  google::ParseCommandLineFlags(&argc, &argv, true);
  google::InitGoogleLogging(argv[0]);

  sql::mysql::MySQL_Driver* driver = sql::mysql::get_mysql_driver_instance();
  LOG(INFO) << "Connecting to database (" << FLAGS_database << ") at " << FLAGS_hostname;
  sql::Connection* con = driver->connect("tcp://" + FLAGS_hostname, 
					 FLAGS_database_user, 
					 FLAGS_database_password);
  
  if (!con) {
    LOG(ERROR) << "Could not connect to database (" << FLAGS_database << ") at " << FLAGS_hostname;
    exit(1);
  }

  sql::Statement* stmt = con->createStatement();

  LOG(INFO) << "Pulling records from table: " << FLAGS_database << ":" << FLAGS_table;
  stmt->execute("USE " + FLAGS_database);
  sql::ResultSet* records = stmt->executeQuery("SELECT id, panoid "
					       "FROM " + FLAGS_table + " "
					       "WHERE expid='" + FLAGS_experiment + "'");

  TrainingData training_data;
  while (records->next()) {
    string id = records->getString("id");
    string panoid = records->getString("panoid");
    // Get all of the annotations for this experiment.
    sql::ResultSet* annotations = stmt->executeQuery("SELECT x, y, width, height "
						     "FROM " + FLAGS_annotations_table + " "
						     "WHERE test_id=" + id);
    while (annotations->next()) {
      Annotations annotation;
      annotation.x = annotations->getInt("x");
      annotation.y = annotations->getInt("y");
      annotation.width = annotations->getInt("width");
      annotation.height = annotations->getInt("height");

      training_data[panoid].push_back(annotation);
    }
    delete annotations;
  }
  delete records;

  delete stmt;
  delete con;

  SaveTrainingData(training_data, FLAGS_output_file);
  return 0;
}
